@@ -1,34 +1,37 @@
 /* exprtype.c -- propagates datatype thru expressions.
 
-Copyright (c) 1998 by Robert K. Moniot.
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the "Software"),
-to deal in the Software without restriction, including without limitation
-the rights to use, copy, modify, merge, publish, distribute, sublicense,
-and/or sell copies of the Software, and to permit persons to whom the
-Software is furnished to do so, subject to the following conditions:
+Copyright (c) 2001 by Robert K. Moniot.
 
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
+Permission is hereby granted, free of charge, to any person
+obtaining a copy of this software and associated documentation
+files (the "Software"), to deal in the Software without
+restriction, including without limitation the rights to use,
+copy, modify, merge, publish, distribute, sublicense, and/or
+sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following
+conditions:
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
-ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
-OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-OTHER DEALINGS IN THE SOFTWARE.
+The above copyright notice and this permission notice shall be
+included in all copies or substantial portions of the
+Software.
 
-Except as contained in this notice, the name of ftnchek shall not be used
-in advertising or otherwise to promote the sale, use or other dealings in
-this Software without prior written authorization from the author.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
+COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+Acknowledgement: the above permission notice is what is known
+as the "MIT License."
 */
 
 /* I. */
 
-/*  exprtype.c:
+/* $Id: exprtype.c,v 1.12 2001/10/07 22:58:04 moniot Exp $
 
 	Routines to propagate datatype through expressions.
 
@@ -53,17 +56,12 @@ this Software without prior written authorization from the author.
 
 
 PROTO(PRIVATE char* sized_typename,( int type, long size ));
-PROTO(PRIVATE void report_mismatch,( Token *term1, Token *operator, Token *term2 ));
-PROTO(PRIVATE void report_type,( Token *t ));
+PROTO(PRIVATE void report_mismatch,( const Token *term1, const Token *op, const Token *term2 ));
+PROTO(PRIVATE void report_type,( const Token *t ));
 PROTO(PRIVATE int int_power,( int x, int n ));
 PROTO(PRIVATE int eval_intrins,( IntrinsInfo *defn, Token *args ));
 
 
-
-extern int in_assignment_stmt;	/* shared with fortran.y */
-
-#define max(x,y) ((y)>(x)?(y):(x))
-
 	/* shorthand for datatypes.  must match those in symtab.h */
 	/* N.B. Also, the fact that type_DEFAULT=0 is assumed in size
 	   propagation code. */
@@ -129,7 +127,7 @@ PRIVATE unsigned char assignment_type[NumT][NumT]={
 };
 
 
-#define INTRINS_ARGS (op == ',') /* Flag to modify behavior of binexpr_type */
+#define INTRINS_ARGS (opclass == ',') /* Flag to modify behavior of binexpr_type */
 
 	/* Routine used in printing diagnostics: returns string "type" for
 	   unsized objects, "type*size" for explicitly sized things.  Due
@@ -210,13 +208,13 @@ init_typesizes(VOID)
 
 void
 #if HAVE_STDC
-binexpr_type(Token *term1, Token *operator, Token *term2, Token *result)
+binexpr_type(Token *term1, Token *op, Token *term2, Token *result)
 #else /* K&R style */
-binexpr_type(term1,operator,term2,result)
-	Token *term1, *operator, *term2, *result;
+binexpr_type(term1,op,term2,result)
+	Token *term1, *op, *term2, *result;
 #endif /* HAVE_STDC */
 {
-    int	op = operator->class,
+    int	opclass = op->tclass,
 	type1 = datatype_of(term1->TOK_type),
 	type2 = datatype_of(term2->TOK_type),
 	result_type;
@@ -239,7 +237,7 @@ binexpr_type(term1,operator,term2,result)
 		result_type = E;
     }
     else {
-	switch(op) {
+	switch(opclass) {
 				/* arithmetic operators: use lookup table */
 	    case '+':
 	    case '-':
@@ -293,7 +291,7 @@ binexpr_type(term1,operator,term2,result)
 
 	    default:
 		oops_message(OOPS_NONFATAL,
-			     operator->line_num,operator->col_num,
+			     op->line_num,op->col_num,
 			     "operator unknown: type not propagated");
 		result_type = type1;
 		break;
@@ -301,7 +299,7 @@ binexpr_type(term1,operator,term2,result)
 
 	if( (type1 != E && type2 != E) ) {
 	    if( result_type == E) {
-		syntax_error(operator->line_num,operator->col_num,
+		syntax_error(op->line_num,op->col_num,
 			"type mismatch");
 		if(INTRINS_ARGS) {
 		  msg_tail("between intrinsic function arguments:");
@@ -309,15 +307,15 @@ binexpr_type(term1,operator,term2,result)
 		else {
 		  msg_tail("in expression:");
 		}
-		report_mismatch(term1,operator,term2);
+		report_mismatch(term1,op,term2);
 	    }
 	    else if(result_type >= W) {	/* W result */
 				/* F90 warning suppressed for numeric exprs */
 	      if(f77_mixed_expr ||
 		 (f90_mixed_expr && ((type1>=L || type2>=L) || I_logop_I)) ) {
-		nonstandard(operator->line_num,operator->col_num,f90_mixed_expr,0);
+		nonstandard(op->line_num,op->col_num,f90_mixed_expr,0);
 		msg_tail(": incompatible type combination in expression:");
-		report_mismatch(term1,operator,term2);
+		report_mismatch(term1,op,term2);
 	      }
 	      result_type -= W;
 	    }
@@ -327,12 +325,12 @@ binexpr_type(term1,operator,term2,result)
 				   fun_or_substr_handle production if one of
 				   operands is a substring expression.  */
 	    else if( f77_mixed_expr &&
-		     op == tok_concat &&
+		     opclass == tok_concat &&
 		     !is_true(IN_ASSIGN,
-			 (term1->TOK_flags|operator->TOK_flags|term2->TOK_flags)) &&
+			 (term1->TOK_flags|op->TOK_flags|term2->TOK_flags)) &&
 		 ((size1==size_ADJUSTABLE && !is_true(CONST_EXPR,term1->TOK_flags))
 	       || (size2==size_ADJUSTABLE && !is_true(CONST_EXPR,term2->TOK_flags))) ) {
-		nonstandard(operator->line_num,operator->col_num,0,0);
+		nonstandard(op->line_num,op->col_num,0,0);
 		msg_tail(": adjustable size cannot be concatenated here");
 	    }
 	}
@@ -341,9 +339,9 @@ binexpr_type(term1,operator,term2,result)
 				/* Figure out the size of result */
     result_size = size_DEFAULT;
     if(result_type != E &&	/* Error type gets DEFAULT size */
-       op != tok_relop) {	/* Result of compare gets DEFAULT size */
+       opclass != tok_relop) {	/* Result of compare gets DEFAULT size */
 
-      if(op == tok_concat) {	/* string//string yields sum of lengths */
+      if(opclass == tok_concat) {	/* string//string yields sum of lengths */
 	if(size1 == size_UNKNOWN || size2 == size_UNKNOWN)
 	  result_size = size_UNKNOWN;
 	else
@@ -352,7 +350,7 @@ binexpr_type(term1,operator,term2,result)
 	  else {
 	    result_size = size1 + size2;
 	    if(port_long_string && result_size > 255)
-	    nonportable(operator->line_num,operator->col_num,
+	    nonportable(op->line_num,op->col_num,
 			"character expression length exceeds 255");
 	  }
 
@@ -407,9 +405,9 @@ if(debug_latest)
 	if(tc1 == tc2) {/* same type category */
 				/* Intrins args: size promotion illegal */
 	  if(INTRINS_ARGS && ls1 != ls2) {
-	    syntax_error(operator->line_num,operator->col_num,
+	    syntax_error(op->line_num,op->col_num,
 			 "precision mismatch in intrinsic argument list:");
-	    report_mismatch(term1,operator,term2);
+	    report_mismatch(term1,op,term2);
 	  }
 				/* Give -port warning if e.g. plain I+I*2
 				   (variables only) */
@@ -418,13 +416,13 @@ if(debug_latest)
 	        && !is_true(CONST_EXPR,term1->TOK_flags)
 	        && !is_true(CONST_EXPR,term2->TOK_flags))
 	    {
-	      nonportable(operator->line_num,operator->col_num,
+	      nonportable(op->line_num,op->col_num,
 			  INTRINS_ARGS?"intrinsic argument list":"expr");
 	      msg_tail("mixes default and explicit");
 	      msg_tail((is_numeric_type(t1)&&is_numeric_type(t2))?
 			 "precision":"size");
 	      msg_tail("operands:");
-	      report_mismatch(term1,operator,term2);
+	      report_mismatch(term1,op,term2);
 	    }
 	  }
 
@@ -480,9 +478,9 @@ if(debug_latest)
 		     /* Second clause checks R+C and C+C */
 	     || (type_category[result_type] == C &&
 		 (type_category[t1] == R? ls2 != 2*ls1 : ls2 != ls1)) ){
-	    warning(operator->line_num,operator->col_num,
+	    warning(op->line_num,op->col_num,
 		    "promotion may not give desired precision:");
-	    report_mismatch(term1,operator,term2);
+	    report_mismatch(term1,op,term2);
 	  }
 	}
 
@@ -493,7 +491,7 @@ if(debug_latest)
 #ifdef DEBUG_EXPRTYPE
 if(debug_latest) {
 (void)fprintf(list_fd,"\nsize of %s %c",sized_typename(type1,size1),
-	ispunct(op)?op:'~');
+	ispunct(opclass)?opclass:'~');
 (void)fprintf(list_fd," %s = ",sized_typename(type2,size2));
 (void)fprintf(list_fd,"%s",sized_typename(result_type,result_size));
 }
@@ -507,7 +505,7 @@ if(debug_latest) {
 		/* Keep track of constant expressions */
     if( is_true(CONST_EXPR,term1->TOK_flags)
 	 && is_true(CONST_EXPR,term2->TOK_flags)
-         && !(op==tok_power && type2!=I) ) { /* exclude **REAL */
+         && !(opclass==tok_power && type2!=I) ) { /* exclude **REAL */
 		make_true(CONST_EXPR,result->TOK_flags);
     }
 
@@ -552,7 +550,7 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 
 		/* Remember if integer division was used */
     if(result_type == type_INTEGER &&
-	   (op == '/' ||
+	   (opclass == '/' ||
 	    (is_true(INT_QUOTIENT_EXPR,term1->TOK_flags) ||
 	     is_true(INT_QUOTIENT_EXPR,term2->TOK_flags))) ) {
 		make_true(INT_QUOTIENT_EXPR,result->TOK_flags);
@@ -567,17 +565,24 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 	if(r == type_LOGICAL)		/* relational tests are equivalent */
 	    r = arith_expr_type[type1][type2];		/* to subtraction */
 
-	if(op == tok_power && is_true(INT_QUOTIENT_EXPR,term2->TOK_flags) ) {
-	  if(trunc_int_div_exponent)
-	    warning(operator->line_num,operator->col_num,
-			"integer quotient expr used in exponent");
+	if(opclass == tok_power && is_true(INT_QUOTIENT_EXPR,term2->TOK_flags) ) {
+	  if(trunc_int_div_exponent) {
+	    warning(op->line_num,op->col_num,
+		    "integer quotient expr");
+	    msg_expr_tree(term2);
+	    msg_tail("used in exponent");
+	  }
 	  if( ! is_true(INT_QUOTIENT_EXPR,term1->TOK_flags) )
 		make_false(INT_QUOTIENT_EXPR,result->TOK_flags);
 	}
 	else if( r == type_REAL || r == type_DP || r == type_COMPLEX) {
-	  if(trunc_int_div_real)
-	    warning(operator->line_num,operator->col_num,
-	    		"integer quotient expr converted to real");
+	  if(trunc_int_div_real) {
+	    warning(op->line_num,op->col_num,
+		    "integer quotient expr");
+	    msg_expr_tree(is_true(INT_QUOTIENT_EXPR,term1->TOK_flags)?
+				  term1:term2);
+	    msg_tail(" converted to real");
+	  }
 	}
     }
 
@@ -593,7 +598,7 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 	  int a = int_expr_value(term1),
 	      b = int_expr_value(term2),
 	      c;
-	  switch(op) {
+	  switch(opclass) {
 	    case '+': c = a+b; break;
 	    case '-': c = a-b; break;
 	    case '*': c = a*b; break;
@@ -613,7 +618,7 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 	    case tok_NEQV: c = a^b; break;
 	    default:
 	      oops_message(OOPS_NONFATAL,
-			   operator->line_num,operator->col_num,
+			   op->line_num,op->col_num,
 			   "invalid int expr operator");
 			c = 0; break;
 	  }
@@ -624,8 +629,8 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 				/* Integer division (including i**neg)
 				   that yields 0 is suspicious.  */
 	  if(trunc_int_div_zero)
-	    if(c==0 && (op=='/' || op==tok_power)) {
-	      warning(operator->line_num,operator->col_num,
+	    if(c==0 && (opclass=='/' || opclass==tok_power)) {
+	      warning(op->line_num,op->col_num,
 	    		"integer const expr yields result of 0");
 	    }
 	}
@@ -633,10 +638,10 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 				/* Also nonconstant**neg is 0 unless
 				   nonconstant=1 */
       else if(trunc_int_neg_power)
-	if(result_type == type_INTEGER && op == tok_power
+	if(result_type == type_INTEGER && opclass == tok_power
 	      && is_true(EVALUATED_EXPR,term2->TOK_flags)
 	      && int_expr_value(term2) < 0) {
-	  warning(operator->line_num,operator->col_num,
+	  warning(op->line_num,op->col_num,
 		  "integer to negative power usually yields 0");
 	}
   }/* end if !INTRINS_ARGS */
@@ -647,13 +652,13 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 
 void
 #if HAVE_STDC
-unexpr_type(Token *operator, Token *term1, Token *result)
+unexpr_type(Token *op, Token *term1, Token *result)
 #else /* K&R style */
-unexpr_type(operator,term1,result)
-	Token *term1, *operator, *result;
+unexpr_type(op,term1,result)
+	Token *term1, *op, *result;
 #endif /* HAVE_STDC */
 {
-   int	op = operator->class,
+   int	opclass = op->tclass,
 	type1 = datatype_of(term1->TOK_type),
 	result_type;
 
@@ -664,7 +669,7 @@ unexpr_type(operator,term1,result)
 		result_type = E;
     }
     else {
-	switch(op) {
+	switch(opclass) {
 			/* arith operators: use diagonal of lookup table */
 	    case '+':
 	    case '-':
@@ -685,7 +690,7 @@ unexpr_type(operator,term1,result)
 
 	    default:
 		oops_message(OOPS_NONFATAL,
-			     operator->line_num,operator->col_num,
+			     op->line_num,op->col_num,
 			     "unary operator type not propagated");
 		result_type = type1;
 		break;
@@ -693,17 +698,17 @@ unexpr_type(operator,term1,result)
 
 	if( type1 != E ) {
 	    if( result_type == E) {
-		syntax_error(operator->line_num,operator->col_num,
+		syntax_error(op->line_num,op->col_num,
 			"expression incompatible with operator:");
-		msg_tail(operator->src_text);
+		msg_tail(op->src_text);
 		msg_tail("used with");
 		report_type(term1);
 	    }
 	    else if(result_type >= W) {
 	      if(f77_mixed_expr || f90_mixed_expr) {
-		nonstandard(operator->line_num,operator->col_num,f90_mixed_expr,0);
+		nonstandard(op->line_num,op->col_num,f90_mixed_expr,0);
 		msg_tail(": incompatible type used with operator:");
-		msg_tail(operator->src_text);
+		msg_tail(op->src_text);
 		msg_tail("used with");
 		report_type(term1);
 	      }
@@ -733,12 +738,12 @@ unexpr_type(operator,term1,result)
 	if(result_type == type_INTEGER) {	/* Only ints propagated */
 	  int a = int_expr_value(term1),
 	      c;
-	  switch(op) {
+	  switch(opclass) {
 	    case '+': c = a; break;
 	    case '-': c = -a; break;
 	    case tok_NOT: c = ~a; break;
 	    default: oops_message(OOPS_NONFATAL,
-			     operator->line_num,operator->col_num,
+			     op->line_num,op->col_num,
 			     "invalid int expr operator");
 			c = 0; break;
 	  }
@@ -855,7 +860,23 @@ assignment_stmt_type(term1,equals,term2)
 		    promotion = (ls1 > ls2*2);
 		  }
 		  else {
-		    size_trunc = (ls1 < ls2);
+				/* Suppress size truncation warning if rhs
+				   is a literal constant that is sure to fit.
+				   For logicals this is always the case; for
+				   integers we use a suitable threshold.
+				 */
+		    if( (size_trunc = (ls1 < ls2)) &&
+			is_true(LIT_CONST,term2->TOK_flags) ){
+			switch(tc2) {
+			  case L:
+			      size_trunc = FALSE;
+			      break;
+			  case I:
+			      if( term2->value.integer <= SMALL_INT_VALUE )
+				  size_trunc = FALSE;
+			      break;
+			}
+		    }
 		    promotion = ((tc2 == R || tc2 == C) && (ls1 > ls2));
 		  }
 				/* mixed size: default size assigned to
@@ -927,9 +948,12 @@ if(debug_latest) {
 
 	int r=result_type;
 
-	if( r == type_REAL || r == type_DP || r == type_COMPLEX)
+	if( r == type_REAL || r == type_DP || r == type_COMPLEX) {
 	    warning(equals->line_num,equals->col_num,
-			"integer quotient expr converted to real");
+			"integer quotient expr");
+	    msg_expr_tree(term2);
+	    msg_tail(" converted to real");
+	}
       }
 
 
@@ -1014,9 +1038,9 @@ func_ref_expr(id,args,result)
 		/* referencing function makes it no longer a class_SUBPROGRAM
 		   but an expression. */
 #ifndef TOK_type
-	result->class = id->class;
+	result->tclass = id->tclass;
 #endif
-	result->subclass = 0;
+	result->tsubclass = 0;
 	result->TOK_type = type_byte(class_VAR,rettype);
 #ifndef TOK_flags
 	result->TOK_flags = 0;	/* clear all flags */
@@ -1081,9 +1105,9 @@ primary_id_expr(id,primary)
 	symt = hashtab[id->value.integer].loc_symtab;
 	id_type=get_type(symt);
 #ifndef TOK_type
-	primary->class = id->class;
+	primary->tclass = id->tclass;
 #endif
-	primary->subclass = 0;
+	primary->tsubclass = 0;
 	primary->TOK_type = type_byte(storage_class_of(symt->type),id_type);
 #ifndef TOK_flags
 	primary->TOK_flags = 0;
@@ -1102,6 +1126,9 @@ primary_id_expr(id,primary)
 		else {
 		    make_true(LVALUE_EXPR,primary->TOK_flags);
 		}
+		if(symt->active_do_var) {
+		    make_true(DO_VARIABLE,primary->TOK_flags);
+		}
 		if(symt->array_var)
 		    make_true(ARRAY_ID_EXPR,primary->TOK_flags);
 		if(symt->set_flag || symt->common_var || symt->parameter
@@ -1192,7 +1219,7 @@ intrins_arg_cmp(defn,t)
 				   not arrayness.  */
 void
 #if HAVE_STDC
-stmt_fun_arg_cmp(Lsymtab *symt, Token *d_arg, Token *a_arg)
+stmt_fun_arg_cmp(const Lsymtab *symt, const Token *d_arg, const Token *a_arg)
 #else /* K&R style */
 stmt_fun_arg_cmp(symt,d_arg,a_arg)
      Lsymtab *symt;
@@ -1221,7 +1248,7 @@ stmt_fun_arg_cmp(symt,d_arg,a_arg)
 
   if(d_size < 0 || a_size < 0) { /* char size_ADJUSTABLE or UNKNOWN */
     d_size = a_size = size_DEFAULT;	/* suppress warnings on size */
-    d_defsize = a_defsize = TRUE;
+    d_defsize = a_defsize = TRUE; /* these are not used at present */
   }
 
   if(d_class != a_class || d_cmptype != a_cmptype ||
@@ -1241,14 +1268,14 @@ stmt_fun_arg_cmp(symt,d_arg,a_arg)
 				   two terms and their operator */
 PRIVATE void
 #if HAVE_STDC
-report_mismatch(Token *term1, Token *operator, Token *term2)
+report_mismatch(const Token *term1, const Token *op, const Token *term2)
 #else /* K&R style */
-report_mismatch(term1,operator,term2)
-     Token *term1,*operator,*term2;
+report_mismatch(term1,op,term2)
+     Token *term1,*op,*term2;
 #endif /* HAVE_STDC */
 {
   report_type(term1);
-  msg_tail(operator->src_text);
+  msg_tail(op->src_text);
   report_type(term2);
 }
 				/* Routine to document the type
@@ -1256,7 +1283,7 @@ report_mismatch(term1,operator,term2)
 				   has one. */
 PRIVATE void
 #if HAVE_STDC
-report_type(Token *t)
+report_type(const Token *t)
 #else /* K&R style */
 report_type(t)
      Token *t;
@@ -1265,10 +1292,14 @@ report_type(t)
   msg_tail(sized_typename((int)datatype_of(t->TOK_type),t->size));
   if(is_true(ID_EXPR,t->TOK_flags))
     msg_tail(hashtab[t->value.integer].name);
-  else if(is_true(LIT_CONST,t->TOK_flags))
+  else if(is_true(LIT_CONST,t->TOK_flags)) {
     msg_tail("const");
-  else
+    msg_expr_tree(t);
+  }
+  else {
     msg_tail("expr");
+    msg_expr_tree(t);
+  }
 }
 
 
@@ -1387,13 +1418,13 @@ eval_intrins(defn,args)
      Token *args;
 #endif /* HAVE_STDC */
 {
-    intrins_flags_t index;
-    index = (defn->intrins_flags & I_EVALUATED);
+    intrins_flags_t fun_num;
+    fun_num = (defn->intrins_flags & I_EVALUATED);
 
 				/* Args must be evaluated, except for LEN */
-    if( (is_true(EVALUATED_EXPR,args->TOK_flags) || index==I_LEN) &&
-       index > 0 && index < (sizeof(ii_fun)/sizeof(ii_fun[0])) ) {
-      return (*ii_fun[index])(args);
+    if( (is_true(EVALUATED_EXPR,args->TOK_flags) || fun_num==I_LEN) &&
+       fun_num > 0 && fun_num < (sizeof(ii_fun)/sizeof(ii_fun[0])) ) {
+      return (*ii_fun[fun_num])(args);
     }
     else {
 #ifdef DEBUG_EXPRTYPE
