@@ -1,46 +1,49 @@
-/* ftnchek.h:
+/* $Id: ftnchek.h,v 1.76 2001/11/03 00:55:37 moniot Exp $
 
 	Common definitions for Fortran Program Checker
 */
 
 /*
 
-Copyright (c) 1999 by Robert K. Moniot.
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the "Software"),
-to deal in the Software without restriction, including without limitation
-the rights to use, copy, modify, merge, publish, distribute, sublicense,
-and/or sell copies of the Software, and to permit persons to whom the
-Software is furnished to do so, subject to the following conditions:
+Copyright (c) 2001 by Robert K. Moniot.
 
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
+Permission is hereby granted, free of charge, to any person
+obtaining a copy of this software and associated documentation
+files (the "Software"), to deal in the Software without
+restriction, including without limitation the rights to use,
+copy, modify, merge, publish, distribute, sublicense, and/or
+sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following
+conditions:
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
-ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
-OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-OTHER DEALINGS IN THE SOFTWARE.
+The above copyright notice and this permission notice shall be
+included in all copies or substantial portions of the
+Software.
 
-Except as contained in this notice, the name of ftnchek shall not be used
-in advertising or otherwise to promote the sale, use or other dealings in
-this Software without prior written authorization from the author.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
+COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+Acknowledgement: the above permission notice is what is known
+as the "MIT License."
 Acknowledgement: the above permission notice was copied, with minor
 changes, from the XFree86 Project.
 
 */
 
 #define COPYRIGHT_DATE \
-"Copyright (C) 1994, 1997, 1999 by Robert K. Moniot"
+"Copyright (C) 1994, 1997, 2000 by Robert K. Moniot"
 #define COPYRIGHT_NOTICE \
 "This program is free software.  Permission is granted to\n\
 modify it and/or redistribute it, retaining this notice.\n\
 No guarantees accompany this software."
-#define VERSION_NUMBER		"FTNCHEK Version 2.11 July 1999"
+#define VERSION_NUMBER		"FTNCHEK Version 3.1 May 2001"
 #define PATCHLEVEL		"Patch Level 2"
 #define PROJECT_VERSION		"P3" /* Project file format version number */
 
@@ -69,26 +72,31 @@ No guarantees accompany this software."
 	/* The following system defines should be defined with the -D
 	   (for UNIX) or /DEFINE (for VMS) compiler options in the makefile,
 	   not here.  They are shown here so you know what to define.
+	   MSDOS should be defined for IBM PC version.  We set it here
+	   automatically if a DOS/Win compiler is detected, otherwise when
+	   compiling you should define it on compiler command line.
+	   Only one of MSDOS, VMS, or UNIX should be defined.  They control
+	   details of behavior such as filename extensions and use of slash
+	   as an option prefix.
 	*/
 
-/*#define VMS*/		/* Set flag for VAX/VMS   system-dependent defns. */
-/*#define UNIX*/	/* Set flag for UNIX (ATT or BSD) defns. */
-#ifdef __TURBOC__
-#define MSDOS		/* Set flag for MSDOS (IBM PC) */
-#define T_ALLOC		/* Specify workaround for malloc limits */
+/*#define VMS*/		/* Set flag for VMS system-dependent defns. */
+/*#define UNIX*/	/* Set flag for UNIX (any flavor) defns. */
+	/* Many (all?) Windows compilers autodefine _WIN32. I am guessing
+	   about _MSDOS.  TurboC (DOS and Win versions) sets __TURBOC__ */
+#if defined(_WIN32) || defined(_MSDOS) || defined(__TURBOC__)
+#define MSDOS		/* Set flag for MSDOS (including Windows) */
 #endif
 
-#ifdef MSDOS
-#define NO_FLOATING_POINT	/* No need for floating point coprocessor */
-#endif
 			/* For portability, declare standard routines here. */
-#if defined(__STDC__) || defined(__osf__)
+#if HAVE_STDLIB_H
 #include <stdlib.h>
 #else
 PROTO(char *getenv,(char *));
 PROTO(char *malloc,(unsigned));
 PROTO(char *calloc,(unsigned,unsigned));
-PROTO(void exit,(int));
+PROTO(VOID free,(void *));
+PROTO(VOID exit,(int));
 #endif
 
 
@@ -201,8 +209,14 @@ PROTO(void exit,(int));
 	      (syntax accepted, options supported, etc.)
 **************************************************************************/
 
-
-
+	/* The following macro turns on useful extra information saying
+	   "expecting whatever" in "parse error" messages from GNU
+	   bison-generated parser.  This macro may not do anything if
+	   parser is generated by another version of bison or yacc.
+	*/
+#ifndef NO_YYERROR_VERBOSE
+#define YYERROR_VERBOSE
+#endif
 
 
 			/* This is the default limit on "cascades" of error
@@ -225,6 +239,13 @@ PROTO(void exit,(int));
 #ifndef VCG_GRAPH_OPTIONS
 #define VCG_GRAPH_OPTIONS "color: lightgray\n"
 #endif
+#endif
+
+	/* Define the amount of indentation of declarations produced
+	   under -makedcls in free-form mode.  Must be >= 0.
+	 */
+#ifndef DCL_FREEFORM_INDENT
+#define DCL_FREEFORM_INDENT 2
 #endif
 
 	/* The following defines control the default status of
@@ -253,7 +274,6 @@ PROTO(void exit,(int));
 #define ARRAYCHECK_ALL TRUE
 #endif
 
-
 			/* -common options default to COMCHECK_ALL, except for
 			   volatile, which defaults to FALSE.  Define
 			   LAX_COMCHECK to make -common=none the default.
@@ -347,6 +367,15 @@ PROTO(void exit,(int));
 #endif
 
 
+			/* -style options default to STYLECHECK_ALL, which
+			   should be FALSE to prevent ftnchek from spewing
+			   warnings about perfectly normal code.
+			 */
+#ifndef STYLECHECK_ALL
+#define STYLECHECK_ALL FALSE
+#endif
+
+
 			/* -truncation options default to TRUNC_ALL.  Define
 			   LAX_TRUNCATION to make -trunc=none the default.
 			   Otherwise -trunc=all is default.
@@ -485,6 +514,10 @@ PROTO(void exit,(int));
 				/* Default value of -wrap option */
 #ifndef WRAP_COLUMN
 #define WRAP_COLUMN 79		/* When to wrap error messages to next line */
+#endif
+
+#ifndef DEF_IDLETTER_LIST	/* default nonalnums allowed in identifiers */
+#define DEF_IDLETTER_LIST "$_"
 #endif
 
 		/* The following defines used to be optional, now they
@@ -495,27 +528,30 @@ PROTO(void exit,(int));
 		   by -f77 */
 
 #define ALLOW_CRAY_POINTERS 1
-#define ALLOW_DOLLARSIGNS 1
 #define ALLOW_DO_ENDDO 1
 #define ALLOW_INCLUDE 1
 #define ALLOW_QUOTEMARKS 1
 #define ALLOW_TYPELESS_CONSTANTS 1
-#define ALLOW_UNDERSCORES 1
 #define ALLOW_UNIX_BACKSLASH 1
 #define ALLOW_UNIX_CPP 1
 #define ALLOW_VMS_IO 1
 #define ALLOW_IBM_IO 1
 
-	/* If INLINE_COMMENT_CHAR is not defined, ! comments will cause
-	   ftnchek to choke.  F77_INLINE_COMMENT controls whether it
-	   is handled but warned about.
-	*/
-#ifndef NO_INLINE_COMMENT
-#define INLINE_COMMENT_CHAR '!' /* Inline comments starting with '!' */
-#endif
+#define INLINE_COMMENT_CHAR '!' /* Inline comments start with '!' */
+
 
 #define KEEP_ARG_NAMES 1	/* option was formerly experimental */
 
+
+		/* SMALL_INT_VALUE is an integer that is small enough that
+		   we don't need to worry if it is assigned to a small-size
+		   int, for purposes of catching truncation errors.  We
+		   conservatively use maximum signed 8-bit byte value.
+		 */
+#ifndef SMALL_INT_VALUE
+#define SMALL_INT_VALUE 127
+#endif
+
 #ifdef DEVELOPMENT		/* For maintaining the program */
 #define DEBUG_FORLEX
 #endif
@@ -599,7 +635,13 @@ PROTO(void exit,(int));
 #ifndef PTRSPACESZ
 #define PTRSPACESZ	200  /* SMALL_MACHINE ptrs-to-arraydim per chunk */
 #endif
-
+#ifndef LABTABLE_SIZE
+#define LABTABLE_SIZE 109      /* SMALL_MACHINE initial size of label table */
+#endif
+#ifndef REF_LISTS_SIZE 
+#define REF_LISTS_SIZE 200   /* SMALL_MACHINE Lab_ref_lists per chunk*/
+#endif
+ 
 #else  /* end if SMALL_MACHINE */
 
 #ifdef MEDIUM_MACHINE		/* use these if default is too big */
@@ -640,7 +682,12 @@ PROTO(void exit,(int));
 #ifndef PTRSPACESZ
 #define PTRSPACESZ	400 /* MEDIUM_MACHINE Max number of ptrs to arraydim */
 #endif
-
+#ifndef LABTABLE_SIZE
+#define LABTABLE_SIZE 209      /* MEDIUM_MACHINE initial size of labtable */
+#endif
+#ifndef REF_LISTS_SIZE 
+#define REF_LISTS_SIZE 400  /* MEDIUM_MACHINE Lab_ref_lists per chunk*/
+#endif
 #else		/* Default is now LARGE_MACHINE, OK for most modern machines */
 
 #ifndef HASHSZ		/* must be <= max int  */
@@ -679,7 +726,12 @@ PROTO(void exit,(int));
 #ifndef PTRSPACESZ
 #define PTRSPACESZ	2000 /* LARGE_MACHINE Max number of ptrs to arraydim */
 #endif
-
+#ifndef LABTABLE_SIZE
+#define LABTABLE_SIZE 299      /* LARGE_MACHINE initial size of labtable */
+#endif
+#ifndef REF_LISTS_SIZE 
+#define REF_LISTS_SIZE 2000  /* LARGE_MACHINE Lab_ref_lists per chunk*/
+#endif
 #endif /* end if LARGE_MACHINE else */
 
 #endif/*end if SMALL_MACHINE else*/
@@ -695,10 +747,21 @@ PROTO(void exit,(int));
 #define FALSE 0
 #define TRUE 1
 
-#define NO_COL_NUM ((unsigned)999)/* Impossible column number to suppress
+				/* Make sure line numbers use 32 bits */
+#if SIZEOF_INT >= 4
+typedef unsigned LINENO_t;
+#else
+typedef unsigned long LINENO_t;
+#endif
+typedef unsigned COLNO_t;
+
+
+#define NO_COL_NUM ((COLNO_t)999)/* Impossible column number to suppress
 				 * printing in error messages
 				 */
-#define NO_LINE_NUM ((unsigned)0)/* Ditto for line number to suppress flushing
+#define GLOBAL_NO_COL_NUM ((LINENO_t)998)/* Ditto for global-level warnings */
+
+#define NO_LINE_NUM ((LINENO_t)0)/* Ditto for line number to suppress flushing
 				 * of line if error not in local context
 				 */
 
@@ -717,10 +780,17 @@ PROTO(void exit,(int));
 
 #define PRIVATE static	/* For non-shared functions */
 
-		/* Ftnchek has no need of floating-point calculations,
-		   so to allow it to run on machines without coprocessor,
-		   we neutralize the few floating-point operations.
-		*/
+	/* Except when DEBUG_FORLEX is turned on, ftnchek has no need
+	   of floating-point calculations, so to allow it to run on
+	   machines without coprocessor, we neutralize the few
+	   floating-point operations that there are.  */
+
+#ifndef DEBUG_FORLEX
+#ifndef NO_FLOATING_POINT
+#define NO_FLOATING_POINT
+#endif
+#endif
+
 #ifdef NO_FLOATING_POINT
 typedef long DBLVAL;
 #else
@@ -745,7 +815,13 @@ SHARED char *main_filename;	/* name of file containing main program */
 #endif
 
 
-		/* Declare variables for command line options */
+
+	/*****************************************************************/
+	/*         Declare variables for command line options            */
+	/*****************************************************************/
+
+
+
 #ifdef MAIN
 #define OPT(Type,Name,Value) Type Name=Value
 #else
@@ -774,6 +850,12 @@ OPT(int,call_tree_prune,TRUE); /* do not repeat duplicates, use "see above"  */
 OPT(int,call_tree_sort,TRUE);  /* alphabetized, not program order */
 		/* End of -calltree options */
 
+                /* These options are controlled by -crossref */
+OPT(int,print_xref_list,FALSE);  /* print subprogram cross-references */
+OPT(int,print_com_xrefs,FALSE);  /* print common block cross-refs */
+OPT(int,print_lab_refs,FALSE);   /* print label cross-references */
+                /* End of -crossref options */
+
 		/* These options are controlled by -common */
 OPT(int,comcheck_type,COMCHECK_ALL);/* Check for type mismatch */
 OPT(int,comcheck_length,COMCHECK_ALL);/* Check for length mismatch  */
@@ -787,8 +869,10 @@ OPT(int,f77_20_continue,F77_ALL);
 OPT(int,f77_accept_type,F77_ALL);
 OPT(int,f77_assignment,F77_ALL);
 OPT(int,f77_array_bounds,F77_ALL);
+OPT(int,f77_attrbased_typedecl,F77_ALL);
 OPT(int,f77_automatic_array,F77_ALL);
 OPT(int,f77_byte,F77_ALL);
+OPT(int,f77_case_construct,F77_ALL);
 OPT(int,f77_cray_pointers,F77_ALL);
 OPT(int,f77_common_subprog_name,F77_ALL);
 OPT(int,f77_construct_name,F77_ALL);
@@ -818,6 +902,7 @@ OPT(int,f77_param_noparen,F77_ALL);
 OPT(int,f77_quad_constants,F77_ALL);
 OPT(int,f77_quotemarks,F77_ALL);
 OPT(int,f77_relops,F77_ALL);
+OPT(int,f77_semicolon,F77_ALL);
 OPT(int,f77_stmt_order,F77_ALL);
 OPT(int,f77_typeless_constants,F77_ALL);
 OPT(int,f77_typesize,F77_ALL);
@@ -830,6 +915,7 @@ OPT(int,f77_variable_format,F77_ALL);
 		/* These options are controlled by -f90 */
 OPT(int,f90_accept_type,F90_ALL);
 OPT(int,f90_byte,F90_ALL);
+OPT(int,f90_continue,F90_ALL);
 OPT(int,f90_cray_pointers,F90_ALL);
 OPT(int,f90_d_comment,F90_ALL);
 OPT(int,f90_dec_tabs,F90_ALL);
@@ -851,6 +937,9 @@ OPT(int,f90_typesize,F90_ALL);
 OPT(int,f90_unix_backslash,F90_ALL);
 OPT(int,f90_unix_cpp,F90_ALL);
 OPT(int,f90_variable_format,F90_ALL);
+	/* Abuse of blank space in free format is not treated as a supported
+	   extension to F90 but as a syntax error. Thus it is on by default. */
+#define f90_freeform_space misc_warn
 		/* End of -f90 options */
 
 		/* These options are controlled by -f95 */
@@ -860,6 +949,7 @@ OPT(int,f95_assign,F95_ALL);	/* ASSIGN stmt & kin */
 OPT(int,f95_Hedit,F95_ALL);	/* H edit descriptor */
 		/* End of -f95 options */
 
+
 		/* These options are controlled by -makedcls */
 OPT(int,dcl_declarations,FALSE);
 OPT(int,dcl_only_undeclared,FALSE);
@@ -871,6 +961,7 @@ OPT(int,dcl_excl_sftran3_internal_vars,FALSE);
 OPT(int,dcl_asterisk_comment_character,FALSE);
 OPT(int,dcl_lowercase_comment_character,FALSE);
 OPT(int,dcl_no_array_dimensions,FALSE);
+OPT(int,dcl_free_form,FALSE);
 		/* End of -makedcls options */
 
 		/* These options are controlled by -portability */
@@ -887,6 +978,7 @@ OPT(int,port_tabs,PORT_ALL);		/* Tabs in source */
 
 
 		/* These options are controlled by -pretty */
+OPT(int,pretty_alt_return,PRETTY_ALL);/* alternate return out of range */
 OPT(int,pretty_multiple_common,PRETTY_ALL);/* COMMON decl in multiple stmts */
 OPT(int,pretty_multiple_namelist,PRETTY_ALL);/* NAMELIST decl in multiple stmts */
 OPT(int,pretty_parens,PRETTY_ALL);	/* Parentheses around a variable */
@@ -896,14 +988,54 @@ OPT(int,pretty_no_space,PRETTY_ALL);    /* Space missing btw id and keyword */
 OPT(int,pretty_contin,PRETTY_ALL);	/* Continuation mark after comment */
 		/* End of -pretty options */
 
+		/* These options are controlled by -project */
+OPT(int,make_project_file,FALSE);/* Save symtab defns in .prj file */
+OPT(int,proj_trim_calls,TRUE);  /* Keep min no of calls in project file */
+OPT(int,proj_trim_common,TRUE); /* Keep min no of block decls in proj file */
+		/* End of -project options */
+
+
 		/* These options are controlled by -source */
 OPT(int,source_dec_tab,DEC_TABS); /*DEC Fortran tab-format*/
 OPT(int,source_vms_include,VMS_INCLUDE); /*VMS INCLUDE statement*/
 OPT(int,source_unix_backslash,UNIX_BACKSLASH); /*UNIX backslash escape*/
 OPT(int,source_dec_param_std_type,DEC_PARAMETER_STANDARD_TYPE);
 OPT(int,source_param_implicit,PARAMETER_IMPLICIT_TYPE);/* std params */
+OPT(int,source_fixed_form,FALSE); /* force fixed format (see Note) */
+OPT(int,source_free_form,FALSE); /* force free format (see Note) */
+	/* Note: actual flag variable used to specify source form is
+	   free_form in forlex.h.  It is set by init_scan based on
+	   filename extension (.f or .f90), overridden by
+	   source_fixed_format or source_free_format if set.*/
+
 		/* end of -source options */
 
+		/* These options are controlled by -style */
+		/* require subprog END stmts to specify subprog class */
+OPT(int,style_req_structured_end,STYLECHECK_ALL);
+		/* require names on structured END stmts */
+OPT(int,style_req_end_name,STYLECHECK_ALL);
+		/* require names on IF, DO, and SELECT CASE constructs */
+OPT(int,style_req_construct_name,STYLECHECK_ALL);
+		/* require main prog to start with PROGRAM stmt */
+OPT(int,style_req_prog_stmt,STYLECHECK_ALL);
+		/* require use of block IF or logical IF */
+OPT(int,style_req_block_if,STYLECHECK_ALL);
+		/* require use of ENDDO with DO */
+OPT(int,style_req_enddo,STYLECHECK_ALL);
+		/* require use of ENDDO or CONTINUE with DO */
+OPT(int,style_req_do_construct,STYLECHECK_ALL);
+		/* no shared DO loop terminators */
+OPT(int,style_shared_do_terminator,STYLECHECK_ALL);
+		/* GOTO in all its forms */
+OPT(int,style_goto,STYLECHECK_ALL);
+		/* Labeled stmt other than FORMAT */
+OPT(int,style_labeled_exec,STYLECHECK_ALL);
+		/* Labeled FORMAT stmt */
+OPT(int,style_labeled_format,STYLECHECK_ALL);
+
+		/* End of -style options */
+
 		/* These options are controlled by -truncation */
 OPT(int,trunc_int_div_real,TRUNC_ALL);	/* Int/int --> real */
 OPT(int,trunc_int_div_exponent,TRUNC_ALL);/* Int/int as exponentl */
@@ -925,15 +1057,19 @@ OPT(int,usage_arg_alias_modified,USAGE_ALL);	/* arg same-as other modified */
 OPT(int,usage_array_alias_modified,USAGE_ALL);/* arg in same array as other modified */
 OPT(int,usage_arg_common_modified,USAGE_ALL);	/* arg in common modified */
 OPT(int,usage_array_common_modified,USAGE_ALL);	/* array arg in common modified */
+OPT(int,usage_arg_unused,USAGE_ALL);		/* arg declared but not used */
 OPT(int,usage_com_block_unused,USAGE_ALL);	/* whole block declared but not used */
 OPT(int,usage_com_block_volatile,USAGE_ALL);	/* block may lose defn */
 OPT(int,usage_com_var_set_unused,USAGE_ALL);	/* set but not used */
 OPT(int,usage_com_var_uninitialized,USAGE_ALL);/* used but not set */
 OPT(int,usage_com_var_unused,USAGE_ALL);	/* declared but not used */
+OPT(int,usage_do_var_modified,USAGE_ALL);	/* DO index var modified */
 OPT(int,usage_ext_declared_only,USAGE_ALL);	/* declared EXTERNAL but not defined or used */
 OPT(int,usage_ext_multiply_defined,USAGE_ALL);/* multiple definitions */
 OPT(int,usage_ext_undefined,USAGE_ALL);	/* used but not defined (= -external flag) */
 OPT(int,usage_ext_unused,USAGE_ALL);	/* defined but not used (like -library flag) */
+OPT(int,usage_label_undefined,USAGE_ALL);  /* label used but undefined */
+OPT(int,usage_label_unused,USAGE_ALL);     /* label defined but unused */
 OPT(int,usage_var_set_unused,USAGE_ALL);	/* set but not used */
 OPT(int,usage_var_uninitialized,USAGE_ALL);	/* used before set */
 OPT(int,usage_var_unused,USAGE_ALL);		/* declared but not used */
@@ -947,20 +1083,14 @@ OPT(int,misc_warn,TRUE);
 	/* The following variables are controlled by command line options */
 
 OPT(int,do_check,TRUE);		/* For -nocheck option */
-OPT(int,print_xref_list,FALSE);	/* Print subprogram cross-references */
 OPT(int,decls_required,FALSE);	/* List all undeclared identifiers */
 OPT(int,div_check,FALSE);	/* Check for possible division by zero */
 OPT(int,ext_def_check,TRUE);	/* Check defined status of externals*/
 OPT(int,help_screen,FALSE);	/* Print out help screen */
 OPT(int,library_mode,FALSE);	/* Set used-flag for all modules in file */
-
-#ifdef EOLSKIP
-OPT(int,eol_is_space,TRUE);	/* Treat contd stmt linebreaks as space */
-#endif
-
+OPT(int,brief,FALSE);		/* Briefer form of local error messages */
 OPT(int,do_list,FALSE);		/* Listing flag */
 OPT(int,novice_help,TRUE);	/* Extra help for novices */
-OPT(int,make_project_file,FALSE);/* Save symtab defns in .prj file */
 OPT(int,pure_functions,TRUE);	/* Assume functions are pure */
 OPT(int,quiet,FALSE);		/* Less verbose output format */
 OPT(int,sixclash,FALSE);	/* To check if names unique in 1st 6 chars */
@@ -977,9 +1107,7 @@ OPT(int,show_resources,FALSE);	/* space avail and used */
 #ifdef DEBUG_FORLEX
 OPT(int,debug_lexer,FALSE);	/* list of tokens as scanned */
 #endif
-#ifdef MAIN
-extern int yydebug;		/* grammar debug via yydebug */
-#endif
+
 		/* Declare variables for commandline settings */
 OPT(int,call_tree_options,0);	/* Sum of: 1 (print it), */
 				/* 2 (who-calls-who form), 4 (don't prune) */
@@ -997,7 +1125,7 @@ OPT(int,intrinsic_set_unix,(DEF_INTRINSIC_SET & 2)==2);/* some unix intrinsics *
 OPT(int,intrinsic_set_vms,(DEF_INTRINSIC_SET & 4)==4);/* some vms intrinsics */
 #endif /* not STANDARD_INTRINSICS */
 
-OPT(int,max_stmt_col,72);	/* End of statement field ( <= MAXLINE )*/
+OPT(int,fixed_max_stmt_col,72);	/* End of fixed-form statement field ( <= MAXLINE )*/
 OPT(int,error_cascade_limit,DEF_ERROR_CASCADE_LIMIT);/* max errors in cascade */
 OPT(int,local_ptrsize,PTRSIZE);	/* Size of Cray pointer variables */
 OPT(int,given_ptrsize,PTRSIZE);	/* Size requested for Cray pointer variables */
@@ -1009,8 +1137,6 @@ OPT(int,wrap_column,WRAP_COLUMN);/* For wrapping error messages */
 
 			/* Shorthands for checking control settings */
 
-#define usage_var_any (usage_var_set_unused || usage_var_uninitialized \
-       || usage_var_unused)
 #define usage_com_any (usage_com_block_unused || usage_com_var_set_unused \
        || usage_com_var_uninitialized || usage_com_var_unused)
 
@@ -1018,34 +1144,32 @@ OPT(int,wrap_column,WRAP_COLUMN);/* For wrapping error messages */
 
 #define check_com_tree (comcheck_volatile&&usage_com_block_volatile) /* Check undef errors */
 
+#define full_output	(do_list || do_symtab)
 
+#define ANY_DCL_DECLARATIONS() (dcl_declarations\
+  ||dcl_only_undeclared\
+  ||dcl_compact||dcl_use_continuations\
+  ||dcl_keywords_lowercase\
+  ||dcl_vars_and_consts_lowercase\
+  ||dcl_excl_sftran3_internal_vars\
+  ||dcl_asterisk_comment_character\
+  ||dcl_lowercase_comment_character\
+  ||dcl_no_array_dimensions\
+  ||dcl_free_form\
+)
 
 		/* Declare variables for commandline StrSettings */
-OPT(char,*out_fname,(char *)NULL);	/* Output filename */
+OPT(char*,out_fname,(char *)NULL);	/* Output filename */
 #ifdef ALLOW_INCLUDE
-OPT(char,*include_path,(char *)NULL);	/* An include-file directory */
+OPT(char*,include_path,(char *)NULL);	/* An include-file directory */
 #endif
-#ifndef STANDARD_INTRINSICS
-OPT(char,*intrinsic_option_list,(char *)NULL); /* intrinsic fcn options */
-#endif
-OPT(char,*argcheck_warn_list,(char *)NULL);/* Arg mismatches to warn about */
-OPT(char,*arraycheck_warn_list,(char *)NULL);/* Arg arrayness warnings */
-OPT(char,*calltree_opt_list,(char *)NULL);/* Subprogram call graph options */
-OPT(char,*comcheck_warn_list,(char *)NULL);/* Common block mismatch warnings */
-OPT(char,*f77_warn_list,(char *)NULL); /* Non F77 extensions to warn about */
-OPT(char,*f90_warn_list,(char *)NULL); /* Non F90 extensions to warn about */
-OPT(char,*f95_warn_list,(char *)NULL); /* Non F95 old syntax to warn about */
-OPT(char,*makedcl_warn_list,(char *)NULL); /* Make-declarations options */
-OPT(char,*port_warn_list,(char *)NULL); /* Nonportable things to warn about */
-OPT(char,*pretty_warn_list,(char *)NULL); /* Misleading things to warn about */
-OPT(char,*source_form_list,(char *)NULL); /* Source format options */
-OPT(char,*trunc_warn_list,(char *)NULL); /* Truncation pitfalls to warn about */
-OPT(char,*usage_warn_list,(char *)NULL); /* Variable usages to warn about */
+OPT(char*,idletter_list,DEF_IDLETTER_LIST);/* non-alpha chars allowed in identifiers */
 
-SHARED unsigned
+SHARED LINENO_t
     line_num,		/* line num of current char */
+    next_line_num;	/* line num of lookahead char */
+SHARED COLNO_t
     col_num,		/* column num of current char */
-    next_line_num,	/* line num of lookahead char */
     next_col_num;	/* column num of lookahead char */
 
 SHARED unsigned
@@ -1060,23 +1184,26 @@ SHARED unsigned long
     tot_line_count,	/* total source lines (stmts + comments) */
     tot_stmt_line_count,/* total source stmt lines  */
     tot_exec_stmt_count,/* count of executable stmts in program */
-    tot_module_count,	/* count of modules in program */
+    tot_module_count,	/* count of subprograms in program */
+    tot_label_count,	/* total number of statement labels defined */
     exec_stmt_count,	/* count of executable stmts in module */
     max_exec_stmt_count,/* max number of executable stmts in any module */
-    max_loc_symtab,	/* number of local symtab entries used */
-    max_glob_symtab,	/* number of global symtab entries used */
     max_loc_strings,	/* chars of local stringspace used */
     max_srctextspace,	/* chars of source text space used */
     max_paraminfo,	/* number of param info structs used */
     max_tokenlists,	/* number of tokenlists constructed */
     max_token_space,	/* number of tokens used */
     max_ptrspace,	/* number of pointers used */
+    max_labels,		/* maximum no. of labels in any subprog */
 
     glob_strings_used,		/* chars of global stringspace used */
     arglist_element_used,	/* arg array elements used */
     arglist_head_used,		/* arg heads used (1 per call) */
     comlist_element_used,	/* com array elements used */
     comlist_head_used;		/* com heads used (1 per defn) */
+SHARED long
+    max_loc_symtab,	/* number of local symtab entries used */
+    max_glob_symtab;	/* number of global symtab entries used */
 
 SHARED int
     equivalence_flag,   /* true while parsing EQUIVALENCE statement */
@@ -1088,6 +1215,11 @@ SHARED int
     prev_token_class,	/* token class of last-returned token */
     curr_stmt_class;	/* Token class of current stmt's leading token */
 
+	/* flag for error messages to behave differently in global cases */
+SHARED int doing_wrapup;
+	/* ditto for local cases */
+SHARED int doing_end_proc;
+
 		/* Define linked-list structure for include-path list */
 #ifdef ALLOW_INCLUDE
 typedef struct IPNode {
@@ -1104,28 +1236,45 @@ SHARED IncludePathNode *include_path_list; /* header to the list */
 	/* in exprtype.c */
 PROTO(void init_typesizes, ( void ));
 
+	/* in ftnchek.c */
+PROTO( char * add_ext,( char *s, const char *ext ));
+PROTO( int has_extension,( const char *name, const char *ext ));
+
 	/* in forlex.c */
+PROTO(void make_legal_char_list, ( void ));
 PROTO(void finish_scan, ( void ));
-PROTO(int flush_line_out, ( unsigned n ));
+PROTO(int flush_line_out, ( LINENO_t n ));
 PROTO(void init_keyhashtab, ( void ));
 PROTO(void init_scan, ( void ));
-PROTO(void open_include_file, ( char *fname, unsigned include_line_num ));
+PROTO(void init_stream,( void ));
+
+	/* in include.c */
+PROTO(void open_include_file, ( char *fname, LINENO_t include_line_num ));
+PROTO(int pop_include_file,( void ));
 
 	/* in fortran.y/fortran.c */
 PROTO(void init_parser, ( void ));
 PROTO(int yyparse,  ( void ));
 
-	/* in ftnchek.c */
-PROTO(void msg_tail, ( char *s ));
-PROTO(void nonportable, ( unsigned lineno, unsigned colno, char *s ));
-PROTO(void nonstandard, ( unsigned lineno, unsigned colno, int f90, int f95 ));
-PROTO(void oops_message, ( int severity, unsigned lineno, unsigned colno, char *s ));
-PROTO(void oops_tail, ( char *s ));
-PROTO(void print_a_line, ( FILE *fd, char *line, unsigned num ));
-PROTO(void  syntax_error, ( unsigned lineno, unsigned colno, char *s ));
-PROTO(void ugly_code, ( unsigned lineno, unsigned colno, char *s ));
-PROTO(void warning, ( unsigned lineno, unsigned colno, char *s ));
-PROTO(void yyerror, ( char *s ));
+	/* in message.c */
+PROTO(void msg_tail, ( const char *s ));
+PROTO(void nonportable, ( LINENO_t lineno, COLNO_t colno, const char *s ));
+PROTO(void nonstandard, ( LINENO_t lineno, COLNO_t colno, int f90, int f95 ));
+PROTO(void oops_message, ( int severity, LINENO_t lineno, COLNO_t colno, const char *s ));
+PROTO(void oops_tail, ( const char *s ));
+PROTO(void print_a_line, ( FILE *fd, const char *line, LINENO_t num ));
+PROTO(void syntax_error, ( LINENO_t lineno, COLNO_t colno, const char *s ));
+PROTO(void ugly_code, ( LINENO_t lineno, COLNO_t colno, const char *s ));
+PROTO(void warning, ( LINENO_t lineno, COLNO_t colno, const char *s ));
+PROTO(void local_message, ( const char *filename, LINENO_t lineno, const char *s, const char *tag ));
+PROTO(void global_warning, ( const char *filename, LINENO_t lineno, const char *s ));
+PROTO(void global_message, ( const char *filename, LINENO_t lineno, const char *s ));
+PROTO(void lex_error, (const char *s ));
+PROTO(void yyerror, ( const char *s ));
+PROTO(char* ulongtostr, (unsigned long num));
+		/* Maximum length of a longtostr() result */
+#define MAX_ULONGTOSTR (3*sizeof(unsigned long))
+
 
 	/* in pgsymtab.c */
 PROTO(void check_arglists, ( void ));
@@ -1143,16 +1292,13 @@ PROTO(void proj_file_in, ( FILE *fd ));
 	/* in symtab.c */
 PROTO(void init_tables, ( void ));
 PROTO(void init_globals, ( void ));
-PROTO(unsigned long init_intrins_hashtab, ( void ));
 PROTO(void init_symtab, ( void ));
 PROTO(void note_filename, ( char *s ));
-#ifndef STANDARD_INTRINSICS
-PROTO(void set_intrinsic_options, ( void ));
-#endif
 #ifdef DEBUG_SIZES
 PROTO(void print_sizeofs, ( void ));
 #endif
 
 /* Misc. useful defines */
+#undef MAX
 #define MAX(X,Y) ((X)>(Y)?(X):(Y))
 
