@@ -1,30 +1,36 @@
-/* project.c:
+/*  $Id: project.c,v 1.11 2001/09/14 00:38:17 moniot Exp $
+
+    project.c:
+
 	Project-file I/O routines.
 
-Copyright (c) 1999 by Robert K. Moniot.
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the "Software"),
-to deal in the Software without restriction, including without limitation
-the rights to use, copy, modify, merge, publish, distribute, sublicense,
-and/or sell copies of the Software, and to permit persons to whom the
-Software is furnished to do so, subject to the following conditions:
+Copyright (c) 2001 by Robert K. Moniot.
 
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
+Permission is hereby granted, free of charge, to any person
+obtaining a copy of this software and associated documentation
+files (the "Software"), to deal in the Software without
+restriction, including without limitation the rights to use,
+copy, modify, merge, publish, distribute, sublicense, and/or
+sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following
+conditions:
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
-ROBERT K. MONIOT OR FORDHAM UNIVERSITY BE LIABLE FOR ANY CLAIM, DAMAGES OR
-OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-OTHER DEALINGS IN THE SOFTWARE.
+The above copyright notice and this permission notice shall be
+included in all copies or substantial portions of the
+Software.
 
-Except as contained in this notice, the name of ftnchek shall not be used
-in advertising or otherwise to promote the sale, use or other dealings in
-this Software without prior written authorization from the author.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
+COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+Acknowledgement: the above permission notice is what is known
+as the "MIT License."
 
   Routines included:
 
@@ -49,17 +55,22 @@ this Software without prior written authorization from the author.
 #include "symtab.h"
 #include <string.h>
 
-/* Note: compilation option PROJ_KEEPALL
+/* Two options, proj_trim_calls and proj_trim_common, control whether
+   Ftnchek creates project files with partial or complete global
+   symbol table information.  If these options are TRUE (the usual
+   case), the action is: in library mode, keep only subprogram
+   definitions, those external references not defined in the current
+   file, and only one instance of each common block.  In non-library
+   mode, keep, besides the above, one call of a given routine from
+   each module, and all common block declarations.  Setting
+   proj_trim_calls to FALSE causes all definitions and calls to be
+   kept.  Setting proj_trim_common to FALSE causes all common block
+   instances to be kept.  (In this case the action is the same whether
+   or not in library mode.)  These options formerly were controlled by
+   compile-time define PROJ_KEEPALL.  They are useful mainly for
+   debugging ftnchek and for using the project files for purposes
+   other than ftnchek.  */
 
-   Define the symbol PROJ_KEEPALL to make Ftnchek create project files
-   with complete global symbol table information.  Otherwise, the default
-   action is: in library mode, keep only subprogram definitions, those
-   external references not defined in the current file, and only one
-   instance of each common block.  In non-library mode, the default is to
-   keep, besides the above, one call of a given routine from each module,
-   and all common block declarations.
-   PROJ_KEEPALL is useful mainly for debugging purposes.
-*/
 #define PROJFILE_COOKIE "FTNCHEK_" /* first part of magic cookie */
 
 
@@ -142,8 +153,8 @@ proj_file_out(fd)
 #endif /* HAVE_STDC */
 {
   Gsymtab *sym_list[GLOBSYMTABSZ]; /* temp. list of symtab entries to print */
-  BYTE sym_has_defn[GLOBSYMTABSZ];
-  BYTE sym_has_call[GLOBSYMTABSZ];
+  char sym_has_defn[GLOBSYMTABSZ];
+  char sym_has_call[GLOBSYMTABSZ];
 
   if(fd == NULL)
     return;
@@ -162,14 +173,15 @@ proj_file_out(fd)
 	(alist=glob_symtab[i].info.arglist) != NULL) {
 			/* Look for defns and calls of this guy. */
 
-	if( (sym_has_defn[numexts]=has_defn(alist)) != (BYTE) FALSE )
+	if( (sym_has_defn[numexts]=has_defn(alist)) != FALSE )
 	   numdefns++;
 	if( (sym_has_call[numexts]= (has_call(alist)
-		/* keep only externals not satisfied in this file */
-#ifndef PROJ_KEEPALL
-		    && (!library_mode || !sym_has_defn[numexts])
-#endif
-				  )) != (BYTE) FALSE )
+		/* keep only externals not satisfied in this file unless
+		   -project=no-trim-calls given.
+		 */
+		    && (!proj_trim_calls ||
+			(!library_mode || !sym_has_defn[numexts]))
+				  )) != FALSE )
 	   numcalls++;
 	if(sym_has_defn[numexts] || sym_has_call[numexts])
 	  sym_list[numexts++] = &glob_symtab[i];
@@ -225,12 +237,11 @@ proj_file_out(fd)
       if(storage_class_of(glob_symtab[i].type) == class_COMMON_BLOCK
 	 && (clist=glob_symtab[i].info.comlist) != NULL &&
 	 clist->topfile == top_filename ) {
-#ifndef PROJ_KEEPALL
 			/* No keepall: save only one com decl if -lib mode */
-	if(library_mode)
+	if( proj_trim_common && library_mode)
 	  numdefns++;
 	else
-#endif			/* keepall or -nolib mode: keep all com decls */
+			/* keepall or -nolib mode: keep all com decls */
 	  numdefns += count_com_defns(clist);
 
 	sym_list[numblocks++] = &glob_symtab[i];
@@ -274,14 +285,15 @@ proj_alist_out(gsymt,fd,do_defns,locally_defined)
     while( a != NULL && a->topfile == top_filename) {
 		/* do_defns mode: output only definitions */
      if( (do_defns && a->is_defn) || (!do_defns && !a->is_defn) )
-#ifndef PROJ_KEEPALL
+
 		/* keep only externals not satisfied in this file in -lib
 		   mode, otherwise keep one actual call from each module. */
-    if( a->is_defn
+    if( ! proj_trim_calls || 
+	(a->is_defn
        || !locally_defined
        || (!library_mode && (a->is_call || a->actual_arg)
-	   && a->module != last_calling_module))
-#endif
+	   && a->module != last_calling_module)) )
+
      {
       last_calling_module = a->module;
       if(a->is_defn)
@@ -343,7 +355,7 @@ proj_alist_out(gsymt,fd,do_defns,locally_defined)
 		arg[i].array_var,
 		arg[i].array_element,
 		arg[i].declared_external,
-		0);		/* possible flag for future use */
+		arg[i].active_do_var);
 	NEXTLINE;
       }
      }/* end if(do_defn...)*/
@@ -416,10 +428,8 @@ proj_clist_out(gsymt,fd)
       }
 			/* keepall or -nolib: loop thru all defns.
 			   Otherwise only keep the first. */
-#ifndef PROJ_KEEPALL
-      if(library_mode)
+      if(proj_trim_common && library_mode)
 	break;
-#endif
       c = c->next;
     }/* end while c != NULL */
 }
@@ -448,14 +458,14 @@ PRIVATE int nil(VOID)/* to make lint happy */
 
 #define READ_FIRST_STR(LEADER,STR) (fscanf(fd,LEADER), \
 				    fscanf(fd,"%127s",STR))
-#define READ_STR(LEADER,STR) ((fscanf(fd,LEADER)==0 &&\
-			       fscanf(fd,"%127s",STR)==1)? READ_OK:READ_ERROR)
-#define READ_ARG(LEADER,STR) ((fscanf(fd,LEADER)==0 && fgetc(fd)==' ' &&\
-		    (getstrn(STR,MAXNAME+1,fd)!=NULL)==1)? READ_OK:READ_ERROR)
-#define READ_NUM(LEADER,NUM) ((fscanf(fd,LEADER)==0 &&\
-			       fscanf(fd,"%d",&NUM)==1)? READ_OK:READ_ERROR)
-#define READ_LONG(LEADER,NUM) ((fscanf(fd,LEADER)==0 &&\
-			       fscanf(fd,"%ld",&NUM)==1)? READ_OK:READ_ERROR)
+#define READ_STR(LEADER,STR) ((void)((fscanf(fd,LEADER)==0 &&\
+			       fscanf(fd,"%127s",STR)==1)? READ_OK:READ_ERROR))
+#define READ_ARG(LEADER,STR) ((void)((fscanf(fd,LEADER)==0 && fgetc(fd)==' ' &&\
+		    (getstrn(STR,MAXNAME+1,fd)!=NULL)==1)? READ_OK:READ_ERROR))
+#define READ_NUM(LEADER,NUM) ((void)((fscanf(fd,LEADER)==0 &&\
+			       fscanf(fd,"%d",&NUM)==1)? READ_OK:READ_ERROR))
+#define READ_LONG(LEADER,NUM) ((void)((fscanf(fd,LEADER)==0 &&\
+			       fscanf(fd,"%ld",&NUM)==1)? READ_OK:READ_ERROR))
 #define NEXTLINE {int c;while( (c=fgetc(fd)) != EOF && c != '\n') continue;\
 		    if(c == EOF) READ_ERROR; else ++proj_line_num;}
 
@@ -589,7 +599,7 @@ proj_arg_info_in(fd,filename,is_defn)
 		arg_array_var,
 		arg_array_element,
 		arg_declared_external,
-		arg_future_flag;	/* possible flag for future use */
+		arg_active_do_var;
 
     if(is_defn)
 	READ_STR(" entry",id_name); /* Entry point name */
@@ -647,7 +657,7 @@ id_name,id_class,id_type);
 #ifdef KEEP_ARG_NAMES
       ArgListHeader *prev_ahead;
       ArgListElement *prev_alist;
-      int prev_n;
+      unsigned prev_n;
 #endif
 
       NEXTLINE;
@@ -794,7 +804,7 @@ alist_class,alist_type,alist_line);
 		&arg_array_var,
 		&arg_array_element,
 		&arg_declared_external,
-		&arg_future_flag) != 8) READ_ERROR;
+		&arg_active_do_var) != 8) READ_ERROR;
 
 #ifdef KEEP_ARG_NAMES
 			/* Economize storage by re-using previously allocated
@@ -816,16 +826,16 @@ alist_class,alist_type,alist_line);
 	  alist[iarg].common_block = (Gsymtab *)NULL;
 	}
 	else {
-	  Gsymtab *gsymt;
-	  int h = hash_lookup(arg_common_block);
+	  Gsymtab *g_symt;
+	  int hh = hash_lookup(arg_common_block);
 			/* Block may not have been seen yet since common
 			   decls come at end of project file.  There is a
 			   risk that block could end up in Gsymtab with no
 			   definition, but only if project file is corrupt.
 			 */
-	  if( (gsymt = hashtab[h].com_glob_symtab) == NULL)
-	    gsymt = install_global(h,type_COMMON_BLOCK,class_COMMON_BLOCK);
-	  alist[iarg].common_block = gsymt;
+	  if( (g_symt = hashtab[hh].com_glob_symtab) == NULL)
+	    g_symt = install_global(hh,type_COMMON_BLOCK,class_COMMON_BLOCK);
+	  alist[iarg].common_block = g_symt;
 	}
 	alist[iarg].common_index = arg_common_index;
 	alist[iarg].same_as = (short)arg_same_as;
@@ -836,6 +846,7 @@ alist_class,alist_type,alist_line);
 	alist[iarg].array_var = arg_array_var;
 	alist[iarg].array_element = arg_array_element;
 	alist[iarg].declared_external = arg_declared_external;
+	alist[iarg].active_do_var = arg_active_do_var;
 	NEXTLINE;
 #ifdef DEBUG_PROJECT
  printf("read arg num %d name %s\n",arg_num,arg_name);
