@@ -1,4 +1,4 @@
-/* symtab.h:
+/* $Id: symtab.h,v 1.36 2001/11/03 00:50:53 moniot Exp $
 
 	Shared declarations for symbol-table routines.  Note: uses
 	declarations in ftnchek.h.
@@ -30,6 +30,8 @@ this Software without prior written authorization from the author.
 
 */
 
+#define SYMTAB_H		/* for intrins.h */
+
 #ifdef SYMTAB			/* "home" for variables is symtab.c */
 #define SYM_SHARED
 #else
@@ -125,11 +127,11 @@ this Software without prior written authorization from the author.
 				/* test for numeric types */
 #define is_numeric_type(t) ((unsigned)(t) <= (unsigned)type_DCOMPLEX)
 				/* test for arith, char, or logical type */
-#define is_const_type(t) (((t)>(unsigned)0) && ((t)<=(unsigned)type_STRING))
+#define is_const_type(t) (((unsigned)(t)>(unsigned)0) && ((unsigned)(t)<=(unsigned)type_STRING))
 				/* test for numeric or logical type */
-#define is_num_log_type(t) ((t) <= type_LOGICAL)
+#define is_num_log_type(t) ((unsigned)(t) <= type_LOGICAL)
 				/* test for real/d.p./complex/d.complx type */
-#define is_float_type(t) ((t)>=type_REAL && (t)<=type_DCOMPLEX)
+#define is_float_type(t) ((unsigned)(t)>=type_REAL && (unsigned)(t)<=type_DCOMPLEX)
 
 	/* Type categories equate DoubleP to Real, Double Complex
 	   to Complex, and Hollerith to Int to simplify expression
@@ -172,6 +174,36 @@ unsigned char equiv_type[]
 #endif
 ;
 
+/*
+ * statement types referencing labels
+ */
+
+#define LAB_NO_TYPE 	   0               /* label never defined */           
+#define LAB_SPECIFICATION  1
+#define LAB_FORMAT	   2
+#define LAB_EXECUTABLE	   3
+#define LAB_GOTO	   4
+#define LAB_ASSIGN	   5
+#define LAB_DO		   6
+#define LAB_IO		   7
+#define LAB_CALL           8
+SYM_SHARED
+char *lab_type_name[]
+#ifdef SYMTAB
+ = {
+    "undef",
+    "specif",
+    "format",
+    "exec",
+    "goto",
+    "assign",
+    "do",
+    "I/O",
+    "arg",
+}
+#endif
+;
+
 typedef unsigned char BYTE;
 
 		/* Array of class and type name translations */
@@ -285,11 +317,11 @@ char *implicit_len_text[('Z'-'A'+1)+2];
 
 			/* temporary equivs for future separate fields */
 				/* these are for array-bounds tokens */
-#define TOK_dims class
-#define TOK_elts subclass
+#define TOK_dims tclass
+#define TOK_elts tsubclass
 
-#define TOK_start class
-#define TOK_end  subclass
+#define TOK_start tclass
+#define TOK_end  tsubclass
 
 
 struct tokstruct {
@@ -302,25 +334,27 @@ struct tokstruct {
 	  *left_token,		/* Left child in expr tree */
 	  *next_token;		/* Right child or next in linked list */
 	char *src_text;		/* Original text string of token */
-	long class,subclass;	/* Token category and subcategory */
+	long tclass,tsubclass;	/* Token category and subcategory */
 	long size;		/* sizeof(datatype) */
 	long TOK_type;		/* Storage class & data type of identifier */
 	unsigned TOK_flags:32;	/* Exprtype flags (see defns below) */
-	unsigned line_num:32;	/* Line where token occurred */
-	unsigned col_num : 8;	/* Column where token occurred */
+	LINENO_t line_num;	/* Line where token occurred */
+	COLNO_t col_num;	/* Column where token occurred */
 	unsigned size_is_adjustable : 1;
 	unsigned size_is_expression : 1;
 };
 
 typedef struct tokstruct Token;
 
+#ifdef  YYSTYPE
 #undef  YYSTYPE
+#endif
 #define YYSTYPE Token	/* Type defn for yylval and Yacc stack */
 
 
 
 SYM_SHARED
-unsigned long loc_symtab_top,	/* Next avail spot in local symbol table */
+long loc_symtab_top,	/* Next avail spot in local symbol table */
    glob_symtab_top;		/* Ditto global */
 
 SYM_SHARED
@@ -349,7 +383,7 @@ SYM_SHARED
       extra_ptrspace;
 
 SYM_SHARED
-  unsigned top_file_line_num;
+  LINENO_t top_file_line_num;
 
 SYM_SHARED
   int global_save;	/* module contains SAVE with no list */
@@ -367,6 +401,10 @@ extern char blank_com_name[],
 extern int block_data_number;
 #endif
 
+typedef int LABEL_t;              /* a label (0-99999) */         
+
+#define NO_LABEL ((LABEL_t)-1)    /* label never used/defined */
+
                 /* Symbol table argument list declarations */
 
 typedef union {		/* InfoUnion: misc info about symtab entry */
@@ -392,7 +430,8 @@ typedef struct {	/* ArgListElement: holds subprog argument data */
 		 used_before_set: 1,
 		 array_var: 1,
 		 array_element: 1,
-		 declared_external: 1;
+		 declared_external: 1,
+		 active_do_var: 1;
 } ArgListElement;
 
 
@@ -403,8 +442,8 @@ typedef struct ALHead {	    /* ArgListHeader: head node of argument list */
 	ArgListElement *arg_array;
 	struct gSymtEntry *module;
 	char *filename,*topfile;
+	LINENO_t line_num,top_line_num;
 	unsigned
-	     line_num,top_line_num,
 	     is_defn: 1,
 	     is_call: 1,
 	     external_decl: 1,	/* EXTERNAL decl, not arg list */
@@ -423,12 +462,13 @@ typedef struct {	/* ComListElement: holds common var data */
 	  used:1,
 	  set:1,
 	  used_before_set:1,
-	  assigned:1;
+	  assigned:1,
+	  marked:1;		/* for listing of offenders */
 } ComListElement;
 
 typedef struct CMHead {	/* ComListHeader: head node of common var list */
 	short numargs;
-	unsigned line_num,top_line_num;
+	LINENO_t line_num,top_line_num;
 	ComListElement *com_list_array;
 	struct gSymtEntry *module;
 	char *filename,*topfile;
@@ -444,7 +484,7 @@ typedef struct TLHead {	/* TokenListHeader: head node of token list */
 	Token *tokenlist;
 	struct TLHead *next;
 	char *filename;
-	unsigned line_num, top_line_num;
+	LINENO_t line_num, top_line_num;
 	unsigned
 	  external_decl:1,
 	  actual_arg:1;
@@ -547,6 +587,11 @@ typedef struct lSymtEntry{
 	struct gSymtEntry *common_block;
 	long common_index;
 	long size;		/* Size of object in bytes */
+		/* Object can be referenced in an include file. Next fields
+		   are line numbers within file where object is referred
+		   to, then come indexes into include-file list.  */
+	LINENO_t line_declared, line_set, line_used;
+	short file_declared, file_set, file_used;
 	BYTE  type;		/* Type & storage class: see macros below */
 			/* Flags */
 	unsigned
@@ -556,6 +601,7 @@ typedef struct lSymtEntry{
 	     used_before_set: 1,/* set_flag is not set when used_flag is set */
 	     is_current_module: 1, /* this symtab entry is the main module */
 	     library_module: 1,	/* module was processed in -library mode */
+	     active_do_var: 1,	/* variable is an active DO index */
 	     array_var: 1,	/* variable is dimensioned */
 	     common_var: 1,	/* variable is in common */
 	     entry_point: 1,	/* name of an entry point */
@@ -615,6 +661,37 @@ typedef struct hashEntry {
 		*com_glob_symtab;/* Global symtab entry for common blocks */
 } HashTable;
 
+SYM_SHARED
+    int current_module_hash	/* hashtable index of current module name */
+#ifdef SYMTAB
+ = -1
+#endif
+;
+
+				/* Struct for include-file list */
+typedef struct {
+    char *fname;		/* name of include file */
+    LINENO_t line;		/* line of topfile where included */
+    short footnote;             /* footnote number--for printing
+				   labels */
+} IncFile;
+
+SYM_SHARED
+IncFile* incfile_list
+#ifdef SYMTAB
+ = (IncFile*)NULL
+#endif
+;
+
+SYM_SHARED
+int num_incfiles		/* number of include-files in list */
+#ifdef SYMTAB
+ = 0
+#endif
+;
+
+SYM_SHARED
+short inctable_index;		/* index of current include-file in list */
 
 				/* Struct for chunks of string space */
 typedef struct STSpace {
@@ -654,7 +731,7 @@ typedef struct PSpace {
 
 		/* Macro to zero out symbol table entry */
 
-#define clear_symtab_entry(S) {register int i;\
+#define clear_symtab_entry(S) {register unsigned i;\
 				 for(i=0;i<sizeof(*S);i++)((char*)S)[i]=0;}
 
 
@@ -685,10 +762,10 @@ typedef struct PSpace {
 		   largest item in the list below.
 		 */
 
-#define make_true(flag,x) ((x) |= (flag))		/* x.flag <-- true   */
-#define make_false(flag,x) ((x) &= ~(flag))		/* x.flag <-- false  */
+#define make_true(flag,x) ((x) |= ((unsigned)flag))	/* x.flag <-- true   */
+#define make_false(flag,x) ((x) &= ~((unsigned)flag))	/* x.flag <-- false  */
 #define is_true(flag,x) ((x) & (flag))			/* x.flag == true?   */
-#define copy_flag(flag,x,y)  ((x) |= ((y)&(flag)))	/* x.flag <-- y.flag */
+#define copy_flag(flag,x,y)  ((x) |= ((y)&((unsigned)flag))) /* x.flag <-- y.flag */
 
 #define ID_EXPR			0x1	/* a variable */
 #define LVALUE_EXPR		0x2	/* assignable */
@@ -710,7 +787,9 @@ typedef struct PSpace {
 #define COMMA_FLAG		0x20000/* keeps track of extra or missing
 				   	  commas in exprlists (obsolete) */
 #define NONSTD_USAGE_FLAG	0x40000	/* concentrator for -f77 warnings */
-#define SYNTAX_ERROR_FLAG	0x80000	/* concentrator for syntax errors */
+#define NOT_DO_TERMINAL_STMT    0x80000 /* stmt illegal as end of DO loop */
+#define DO_VARIABLE		0x100000 /* id is active DO index variable */
+#define SYNTAX_ERROR_FLAG	0x200000/* concentrator for syntax errors */
 
 #ifdef DYNAMIC_TABLES		/* tables will be mallocked at runtime */
 SYM_SHARED
@@ -795,24 +874,33 @@ StrSpace srctextspace;/* String space for token source text */
 
 			/* in exprtype.c */
 PROTO(void assignment_stmt_type,( Token *term1, Token *equals, Token *term2 ));
-PROTO(void binexpr_type,( Token *term1, Token *operator, Token *term2, Token
+PROTO(void binexpr_type,( Token *term1, Token *op, Token *term2, Token
 		  *result ));
 PROTO(void func_ref_expr,( Token *id, Token *args, Token *result ));
 PROTO(void primary_id_expr,( Token *id, Token *primary ));
-PROTO(void stmt_fun_arg_cmp,( Lsymtab *symt, Token *d_arg, Token *a_arg ));
+PROTO(void stmt_fun_arg_cmp,( const Lsymtab *symt, const Token *d_arg, const Token *a_arg ));
 PROTO(int substring_size,( Token *id, Token *limits ));
-PROTO(void unexpr_type,( Token *term1, Token *operator, Token *result ));
+PROTO(void unexpr_type,( Token *term1, Token *op, Token *result ));
 PROTO(int intrins_arg_cmp,( IntrinsInfo *defn, Token *t));
 
+			/* in advance.c */
+PROTO(int see_double_colon,( void ));
+
 			/* in forlex.c */
 PROTO(void implied_id_token,( Token *t, char *s ));
 PROTO(int yylex,( void ));
 
+			/* in keywords.c */
+PROTO(char *keytok_name,(int tclass));
+
 			/* in fortran.y/fortran.c */
 PROTO(void check_seq_header,( Token *t ));
 
-			/* in plsymtab.c */
-PROTO(void print_loc_symbols,( int curmodhash ));
+			/* in prlocsym.c */
+PROTO(void print_loc_symbols,( void ));
+
+			/* in makedcls.c */
+PROTO(void make_declarations,( Lsymtab *sym_list[], char *mod_name ));
 
 			/* in symtab.c */
 PROTO(void call_func,( Token *id, Token *arg ));
@@ -825,14 +913,12 @@ PROTO(void def_array_dim,( Token *id, Token *arg ));
 PROTO(void def_com_block,( Token *id, Token *comlist ));
 PROTO(void def_com_variable,( Token *id ));
 PROTO(int def_curr_module,( Token *id ));
+PROTO(void def_do_variable,( Token *id ));
 PROTO(void def_equiv_name,( Token *id ));
 PROTO(void def_ext_name,( Token *id ));
 PROTO(void def_function,( int datatype, long size, char *size_text, Token
 		  *id, Token *args ));
 PROTO(void def_intrins_name,( Token *id ));
-#ifdef CHECK_LABELS
-PROTO(void def_label,( Token *lab ));
-#endif
 PROTO(void def_namelist,( Token *id, Token *list ));
 PROTO(void def_namelist_item,( Token *id ));
 PROTO(void def_parameter,( Token *id, Token *val, int noparen ));
@@ -840,22 +926,23 @@ PROTO(void def_stmt_function,( Token *id, Token *args ));
 PROTO(void do_ASSIGN,( Token *id ));
 PROTO(void do_assigned_GOTO,( Token *id ));
 PROTO(void do_ENTRY,( Token *id, Token *args, int hashno ));
-PROTO(void do_RETURN,( int hashno, Token *keyword ));
+PROTO(int do_RETURN,( int hashno, Token *keyword ));
 PROTO(void equivalence,( Token *id1, Token *id2 ));
 PROTO(DBLVAL float_expr_value,( Token *t ));
-PROTO(int get_size,( Lsymtab *symt, int type ));
-PROTO(char * get_size_text,( Lsymtab *symt, int type ));
-PROTO(int get_type,( Lsymtab *symt ));
+PROTO(int get_size,( const Lsymtab *symt, int type ));
+PROTO(char * get_size_text,( const Lsymtab *symt, int type ));
+PROTO(int get_type,( const Lsymtab *symt ));
 PROTO(unsigned hash_lookup,( char *s ));
 PROTO(Gsymtab* install_global,( int h, int datatype, int storage_class ));
 PROTO(int int_expr_value,( Token *t ));
 PROTO(char * new_global_string,( char *s ));
 PROTO(void free_textvec,( char **p ));
-PROTO(char * new_src_text,( char *s, int len ));
+PROTO(char * new_src_text,( const char *s, int len ));
 PROTO(char * new_src_text_alloc,( int size ));
 PROTO(char * new_tree_text,( Token *t ));
 PROTO(char ** new_textvec,( int n ));
 PROTO(Token * new_token,( void ));
+PROTO(void msg_expr_tree, (const Token *t));
 #ifdef DEBUG_EXPRTREES
 PROTO(void print_src_text,( Token *t ));
 PROTO(void print_expr_tree,( Token *t ));
@@ -864,12 +951,14 @@ PROTO(void print_expr_list,( Token *t ));
 PROTO(void process_lists,( int curmodhash ));
 PROTO(void ref_array,( Token *id, Token *subscrs ));
 PROTO(void ref_namelist,( Token *id, int stmt_class ));
+PROTO(void ref_identifier,( Token *id ));
 PROTO(void ref_variable,( Token *id ));
 PROTO(void save_com_block,( Token *id ));
 PROTO(void save_variable,( Token *id ));
 PROTO(void set_implicit_type,( int type, long size, char *len_text, int c1, int c2 ));
 PROTO(void stmt_function_stmt,( Token *id ));
-PROTO(char * token_name,( Token t ));
+PROTO(char * token_name,( Token *t ));
+PROTO(void undef_do_variable,( int h ));
 PROTO(void use_actual_arg,( Token *id ));
 PROTO(void use_implied_do_index,( Token *id ));
 PROTO(void use_io_keyword,( Token *keyword, Token *value, int stmt_class ));
@@ -878,8 +967,34 @@ PROTO(void use_lvalue,( Token *id ));
 PROTO(void use_parameter,( Token *id ));
 PROTO(void use_variable,( Token *id ));
 
+PROTO(char* typespec, ( int t, int has_size, long size,
+			int has_len, long len));
+				/* The following size is conservative,
+				   to make sure no buffer overruns occur.
+				 */
+		/* Maximum length of a typespec() result.  */
+#define MAX_TYPESPEC (4+4+6*sizeof(long))
+
 				/* in symtab.c (formerly hash.c) */
-PROTO(unsigned long hash,( char *s ));
+PROTO(unsigned long hash,( const char *s ));
 PROTO(unsigned long rehash,( unsigned long hnum ));
 
 
+			/* To stop printing errors after limit is reached,
+			   unless limit is 0.  Increment error count
+			   even if don't print.
+			 */
+#define CASCADE_LIMIT(ERROR_COUNT) (++(ERROR_COUNT) > error_cascade_limit \
+			&& error_cascade_limit > 0)
+
+
+		/* prototypes of label routines */
+void init_labtable(void);
+void print_labels(void);
+void print_label_refs(void); 
+void check_labels(char *mod_name);
+void sort_labtable(void);
+int def_label(Token *t, int type);
+void def_do_label(Token *t);
+void ref_label(Token *t, int type);
+void update_label_resources(void);
